@@ -10,7 +10,7 @@ exec guile-gtk -s $0 $*
 ;; you press the button and exits.
 ;;
 
-(use-modules (gtk gtk))
+(use-modules (toolkits gtk))
 
 (let ((window (gtk-window-new 'toplevel))
       (button (gtk-button-new-with-label "Say Hello")))
@@ -22,8 +22,8 @@ exec guile-gtk -s $0 $*
 			(display "Hello World!")
 			(newline)
 			(gtk-widget-destroy window)))
-  (gtk-widget-show-all window)
-  (gtk-standalone-main window))
+  (gtk-widget-show button)
+  (gtk-widget-show window))
 
 ; Local Variables:
 ; mode: scheme
