@@ -80,7 +80,8 @@ static void
 enter_proxy (gpointer obj, SCM proxy)
 {
   if (proxy_tab == NULL)
-    proxy_tab = g_hash_table_new (gpointer_hash, gpointer_compare);
+    proxy_tab = g_hash_table_new ((GHashFunc)gpointer_hash,
+				  (GCompareFunc)gpointer_compare);
   g_hash_table_insert (proxy_tab, obj, (gpointer)proxy);
 }
 
@@ -159,22 +160,6 @@ typedef struct _type_infos {
 
 static type_infos *all_type_infos;
 
-static GtkType
-gtk_type_register_builtin (char   *name,
-			   GtkType parent)
-{
-  GtkTypeInfo info;
-
-  info.type_name = name;
-  info.object_size = info.class_size = 0;
-  info.class_init_func = NULL;
-  info.object_init_func = NULL;
-  info.arg_set_func = NULL;
-  info.arg_get_func = NULL;
-
-  return gtk_type_unique (parent, &info);
-}
-
 void
 sgtk_register_type_infos (sgtk_type_info **infos)
 {
@@ -186,43 +171,71 @@ sgtk_register_type_infos (sgtk_type_info **infos)
   t->infos = infos;
   t->next = all_type_infos;
   all_type_infos = t;
-
-  while (*infos)
-    {
-      sgtk_type_info *info = *infos;
-      if (info->type != GTK_TYPE_INVALID && info->type != GTK_TYPE_OBJECT)
-	{
-	  GtkType parent_type = info->type;
-	  info->type = gtk_type_from_name (info->name);
-	  if (info->type == GTK_TYPE_INVALID)
-	    {
-#ifdef DEBUG_PRINT
-	      fprintf (stderr, "adding missing type `%s'.\n", info->name);
-#endif
-	      info->type = gtk_type_register_builtin (info->name, parent_type);
-	    }
-	  if (GTK_FUNDAMENTAL_TYPE (info->type) != parent_type)
-	    {
-	      fprintf (stderr, "mismatch for type `%s'.\n", info->name);
-	      abort ();
-	    }
-	  enter_type_info (info);
-	}
-      infos++;
-    }
 }
 
+static int
+sgtk_fillin_type_info (sgtk_type_info *info)
+{
+  if (info->type != GTK_TYPE_OBJECT
+      && info->type != GTK_FUNDAMENTAL_TYPE (info->type)
+      && info->type != GTK_TYPE_INVALID)
+    {
+      GtkType parent_type = info->type;
+      info->type = gtk_type_from_name (info->name);
+      if (info->type == GTK_TYPE_INVALID)
+	{
+	  fprintf (stderr, "unknown type `%s'.\n", info->name);
+	  return 0;
+	}
+      if (GTK_FUNDAMENTAL_TYPE (info->type) != parent_type)
+	{
+	  fprintf (stderr, "mismatch for type `%s'.\n", info->name);
+	  info->type = GTK_TYPE_INVALID;
+	  return 0;
+	}
+      enter_type_info (info);
+    }
+
+  return 1;
+}      
+     
 sgtk_type_info*
 sgtk_find_type_info (GtkType type)
 {
-  sgtk_type_info *info, **infos;
-  type_infos *t;
+  sgtk_type_info *info;
+  type_infos *infos;
   char *name;
 
   info = sgtk_get_type_info (GTK_TYPE_SEQNO(type));
   if (info)
     return info;
 
+  /* XXX - merge this with the GtkObject code.  I don't have the brain
+     right now to do it. */
+
+  name = gtk_type_name (type);
+  for (infos = all_type_infos; infos; infos = infos->next)
+    {
+      sgtk_type_info **ip;
+      for (ip = infos->infos; *ip; ip++)
+	if (!strcmp ((*ip)->name, name))
+	  {
+	    if (GTK_FUNDAMENTAL_TYPE (type) != (*ip)->type)
+	      {
+		fprintf (stderr, "mismatch for type `%s'.\n", name);
+		info->type = GTK_TYPE_INVALID;
+		abort ();
+	      }
+	    (*ip)->type = type;
+	    enter_type_info (*ip);
+	    return *ip;
+	  }
+    }
+
+  /* XXX - should use the Gtk+ type introspection here instead of
+     giving up. */
+
+  fprintf (stderr, "unknown type `%s'.\n", name);
   abort ();
 }
 
@@ -789,6 +802,9 @@ sgtk_boxed2scm (gpointer ptr, sgtk_boxed_info *info, int copyp)
   if (ptr == NULL)
     return SCM_BOOL_F;
 
+  if (!sgtk_fillin_type_info (&info->header))
+    return SCM_BOOL_F;
+
   SCM_DEFER_INTS;
   if (copyp)
     {
@@ -843,19 +859,40 @@ sgtk_float2scm (gfloat f)
   return gh_double2scm ((double)f);
 }
 
+int
+sgtk_valid_double (SCM obj)
+{
+  return SCM_NUMBERP (obj);
+}
+
+SCM gh_double2scm (double);
+
+double
+sgtk_scm2double (SCM obj)
+{
+  return gh_scm2double (obj);
+}
+
+SCM
+sgtk_double2scm (double f)
+{
+  return gh_double2scm (f);
+}
+
 /* Composites. */
 
 int
 sgtk_valid_composite (SCM obj, int (*predicate)(SCM))
 {
-  if (SCM_NIMP(obj) && SCM_CONSP(obj))
+  if (scm_ilength (obj) >= 0)
     {
-      do {
-	if (!predicate (SCM_CAR(obj)))
-	  return 0;
-	obj = SCM_CDR(obj);
-      } while (SCM_NIMP(obj) && SCM_CONSP(obj));
-      return obj == SCM_EOL;
+      while (SCM_NIMP(obj) && SCM_CONSP(obj))
+	{
+	  if (!predicate (SCM_CAR(obj)))
+	    return 0;
+	  obj = SCM_CDR(obj);
+	}
+      return 1;
     }
   else if (SCM_NIMP(obj) && SCM_VECTORP(obj))
     {
@@ -871,7 +908,7 @@ sgtk_valid_composite (SCM obj, int (*predicate)(SCM))
 }
 
 SCM
-sgtk_slist2scm (GSList *list, SCM (*toscm)(gpointer*))
+sgtk_slist2scm (GSList *list, SCM (*toscm)(void*))
 {
   SCM res, *tail = &res;
   while (list)
@@ -885,18 +922,19 @@ sgtk_slist2scm (GSList *list, SCM (*toscm)(gpointer*))
 }
 
 GSList*
-sgtk_scm2slist (SCM obj, void (*fromscm)(SCM, gpointer*))
+sgtk_scm2slist (SCM obj, void (*fromscm)(SCM, void*))
 {
   GSList *res, **tail = &res;
 
-  if (SCM_NIMP(obj) && SCM_CONSP(obj))
+  if (obj == SCM_EOL || (SCM_NIMP(obj) && SCM_CONSP(obj)))
     {
-      do {
-	*tail = g_slist_alloc ();
-	fromscm (SCM_CAR (obj), &(*tail)->data);
-	obj = SCM_CDR(obj);
-	tail = &(*tail)->next;
-      } while (SCM_NIMP(obj) && SCM_CONSP(obj));
+      while (SCM_NIMP(obj) && SCM_CONSP(obj))
+	{
+	  *tail = g_slist_alloc ();
+	  fromscm (SCM_CAR (obj), &(*tail)->data);
+	  obj = SCM_CDR(obj);
+	  tail = &(*tail)->next;
+	}
     }
   else if (SCM_NIMP(obj) && SCM_VECTORP(obj))
     {
@@ -913,6 +951,40 @@ sgtk_scm2slist (SCM obj, void (*fromscm)(SCM, gpointer*))
   return res;
 }
 
+sgtk_cvec
+sgtk_scm2cvec (SCM obj, void (*fromscm)(SCM, void*), size_t sz)
+{
+  sgtk_cvec res;
+  int i, len;
+  char *ptr;
+
+  if ((res.count = scm_ilength (obj)) >= 0)
+    {
+      res.vec = (void *)scm_must_malloc (res.count * sz, "scm2cvec");
+      for (i = 0, ptr = res.vec; i < res.count; i++, ptr += sz)
+	{
+	  fromscm (SCM_CAR (obj), ptr);
+	  obj = SCM_CDR(obj);
+	}
+    }
+  else if (SCM_NIMP(obj) && SCM_VECTORP(obj))
+    {
+      SCM *elts = SCM_VELTS (obj);
+      res.count = SCM_LENGTH (obj);
+      res.vec = (void *)scm_must_malloc (res.count * sz, "scm2cvec");
+      for (i = 0, ptr = res.vec; i < res.count; i++, ptr += sz)
+	fromscm (elts[i], ptr);
+    }
+
+  return res;
+}
+
+void
+sgtk_cvec_free (sgtk_cvec *cvec)
+{
+  scm_must_free (cvec->vec);
+}
+
 /* converting between SCM and GtkArg */
 
 SCM
@@ -937,7 +1009,7 @@ sgtk_arg2scm (GtkArg *a, int free_mem)
     case GTK_TYPE_FLOAT:
       return sgtk_float2scm (GTK_VALUE_FLOAT(*a));
     case GTK_TYPE_DOUBLE:
-      return sgtk_float2scm (GTK_VALUE_DOUBLE(*a));
+      return sgtk_double2scm (GTK_VALUE_DOUBLE(*a));
     case GTK_TYPE_STRING:
       {
 	SCM ret = scm_makfrom0str (GTK_VALUE_STRING(*a));
@@ -993,7 +1065,7 @@ sgtk_scm2arg (GtkArg *a, SCM obj, SCM protector)
       GTK_VALUE_FLOAT(*a) = sgtk_scm2float (obj);
       break;
     case GTK_TYPE_DOUBLE:
-      GTK_VALUE_DOUBLE(*a) = sgtk_scm2float (obj);
+      GTK_VALUE_DOUBLE(*a) = sgtk_scm2double (obj);
       break;
     case GTK_TYPE_STRING:
       SCM_ASSERT (SCM_NIMP(obj) && SCM_STRINGP(obj), obj, SCM_ARG1,
@@ -1057,7 +1129,7 @@ sgtk_scm2ret (GtkArg *a, SCM obj)
       *GTK_RETLOC_FLOAT(*a) = sgtk_scm2float (obj);
       break;
     case GTK_TYPE_DOUBLE:
-      *GTK_RETLOC_DOUBLE(*a) = sgtk_scm2float (obj);
+      *GTK_RETLOC_DOUBLE(*a) = sgtk_scm2double (obj);
       break;
     case GTK_TYPE_STRING:
       SCM_ASSERT (SCM_NIMP(obj) && SCM_STRINGP(obj), obj, SCM_ARG1,
@@ -1102,7 +1174,7 @@ sgtk_scm2ret (GtkArg *a, SCM obj)
 
       (trampoline proc args)
 
-   PROC is the rel callback procedure and ARGS is the list of
+   PROC is the real callback procedure and ARGS is the list of
    arguments that should be passed to it.  */
 
 static SCM callback_trampoline;
@@ -1251,9 +1323,9 @@ sgtk_string_conversion (SCM str)
 /* The SCM_PROC for the exported functions is in gtk-support.c to have
    it be snarfed for sgtk_init_gtk_support. */
 
-static sgtk_object_info *sgtk_find_object_info (char *name);
+sgtk_object_info *sgtk_find_object_info (char *name);
 
-static sgtk_object_info *
+sgtk_object_info *
 sgtk_find_object_info_from_type (GtkType type)
 {
   sgtk_object_info *info;
@@ -1264,7 +1336,7 @@ sgtk_find_object_info_from_type (GtkType type)
   return sgtk_find_object_info (gtk_type_name (type));
 }
 
-static sgtk_object_info *
+sgtk_object_info *
 sgtk_find_object_info (char *name)
 {
   GtkType type, parent;
@@ -1286,6 +1358,9 @@ sgtk_find_object_info (char *name)
       for (ip = infos->infos; *ip; ip++)
 	if (!strcmp ((*ip)->name, name))
 	  {
+	    if (GTK_FUNDAMENTAL_TYPE((*ip)->type) != GTK_TYPE_OBJECT)
+	      return NULL;
+
 	    info = (sgtk_object_info *)*ip;
 	    info->header.type = info->init_func ();
 	    enter_type_info ((sgtk_type_info*)info);
@@ -1344,40 +1419,44 @@ sgtk_find_arg_info (GtkArg *arg, sgtk_object_info *info, char *name)
   char *d = strchr (name, ':');
   if (d && d[1] == ':')
     {
-      /* A long name. */
-      arg->name = name;
-      arg->type = gtk_object_get_arg_type (name);
-      return;
-    }
-  else
-    {
-      /* Search the short names. */
-#ifdef DEBUG_PRINT
-      fprintf (stderr, "searching short `%s'\n", name);
-#endif
+      /* A long name.  Find the object_info for the class part. */
+      int len = d-name;
+
       while (info)
 	{
-	  int i;
-	  for (i = 0; i < info->n_args; i++)
-	    {
-#ifdef DEBUG_PRINT
-	      fprintf (stderr, " on %s\n", info->args[i].name);
-#endif
-	      if (!strcmp (info->args_short_names[i], name))
-		{
-		  *arg = info->args[i];
-		  return;
-		}
-	    }
+	  if (info->header.name[len] == '\0'
+	      && !strncmp (info->header.name, name, len))
+	    break;
 	  info = info->parent;
 	}
+      name = d+2;
     }
-
+  
+#ifdef DEBUG_PRINT
+  fprintf (stderr, "searching short `%s'\n", name);
+#endif
+  while (info)
+    {
+      int i;
+      for (i = 0; i < info->n_args; i++)
+	{
+#ifdef DEBUG_PRINT
+	  fprintf (stderr, " on %s\n", info->args[i].name);
+#endif
+	  if (!strcmp (info->args_short_names[i], name))
+	    {
+	      *arg = info->args[i];
+	      return;
+	    }
+	}
+      info = info->parent;
+    }
+  
   arg->type = GTK_TYPE_INVALID;
   return;
 }
       
-static GtkArg*
+GtkArg*
 sgtk_build_args (sgtk_object_info *info, int *n_argsp, SCM scm_args,
 		 SCM protector)
 {
